/*#   Copyright 2012 http://scms.pro/
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
*/

/*
 * SCMS Hydra Library
 */


#include <netdb.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>


#define CLUSTERCTL	"127.0.0.1"

static int ctlport = 9010;
static int cl;
static int uid;

static const char *hostname = CLUSTERCTL, *user, *task;

#include "utils.h"
#include "privatekey.h"

static void help( void) {

    fprintf( stdout, "hctl -v -i ipaddr -u username command\n");
    exit(0);
}

static void openserver( void) {

    struct sockaddr_in addr;
    struct hostent *hent;
    
    if( !(hent = gethostbyname( hostname))) {
	fprintf( stderr, "cannot get hostaddr: %s\n", hostname);
	exit( 11);
    }

    memset( &addr, 0, sizeof( addr));
    addr.sin_family = AF_INET;
    addr.sin_addr = *(struct in_addr *)*hent->h_addr_list;
    addr.sin_port = htons( ctlport);
    
    if( (cl = socket( AF_INET, SOCK_STREAM, 0)) < 0) {
	perror( "error_socket()");
	exit( 12);
    }
    if( connect( cl, (struct sockaddr*)&addr, sizeof( addr)) < 0) {
	perror( "error_connect()");
	exit( 13);
    }
}

int main( int ac, char **av) {

    char *s, str[ 4194304];
    fd_set fdset;
    struct timeval tv;
    struct sockaddr_in addr;
    struct passwd *pswd;
    int c, len;
    
    int reverse = 0;

    uid = geteuid();

    ac--; av++;
    int ufl=0;
    int ifl=0;
    
    while( ac) {
	s = *av;
	if( *s != '-') break;
        av++; ac--;
        c = s[1];
	switch( c) {
	    case 'v':
		verbose++;
		continue;
	    case 'u':
		user = *av++; ac--;
		ufl=1;
		continue;
	    case 'i':
		hostname = *av++; ac--;
		ifl=1;
		continue;
	    default:
		help();
	}
    }

    if(!ufl || !ifl || !ac) 
	help();
    
    
    openserver();

    if( (read( cl, str, sizeof( str)) != 6) || strcmp( str, "Yes?\n"))
        return 1;

    if( write( cl, "hydra", 6) != 6)
        return 1;
//TODO ADD HANDSHAKE HERE
    if( (len = read_str( cl, str, sizeof( str))) <= 0)
        return 1;
// Cheking against HANDSHAKE1
    if( strcmp( str, HANDSHAKE1))
        return 1;
    if( write_str( cl, HANDSHAKE2, HANDSHAKE2LEN) != len)
        return 1;

    if( !read_ok( cl))
        return 1;

    len = snprintf( str, sizeof( str), "user %s", user);
    if( (write_str( cl, str, len) <= 0) || !read_ok( cl))
        return 1;

  if( ac) {
	  len = 0;
    while( ac) {
	    s = *av++;
	    ac--;
	    len += snprintf( str + len, sizeof( str) - len, ac ? "%s " : "%s", s);
    }
    s = str;
  } else {
    help();
  }
  write_str( cl, s, len);
  if( !read_ok( cl))
    return 1;

  // Write data to pipe first  
  const int readchunk = 4000000;
  int l;

  if ( !isatty(0)) {

    if( !freopen( NULL, "rb", stdin)) {
      perror( "freopen");
      exit(1);
    }

    while( !feof( stdin)) {
      l = fread( str, sizeof(char), readchunk, stdin);
      if( l <= 0)
        break;
      usleep(2);
      if (write_str( cl, str, l) <= 0)
        break;
    }
  }
  write_str( cl, ".DONE.", 0);

  // Then read results of command execution
  while( l = read_str( cl, str, sizeof( str))) {
    if( l <= 0) 
      break;
    usleep(2);
    if( fwrite( str, sizeof(char), l, stdout) <= 0)
      break;
  }
  return 0;
}

