/*#   Copyright 2012 http://scms.pro/
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
*/

/*
 * SCMS Hydra Library
 */


#include <syslog.h>

static int verbose = 0;

static int do_select( int fd, int timeout) {

    fd_set fdset;
    struct timeval tv;
    int t;
    
    t = timeout ? timeout : 5;

    tv.tv_sec = t; tv.tv_usec = 0;
	
    while( 1) {
				    
	FD_ZERO( &fdset);
	FD_SET( fd, &fdset);
			    
        switch( select( fd + 1, &fdset, 0, 0, &tv)) {
            case 0:
		if( !timeout) {
		    tv.tv_sec = t; tv.tv_usec = 0;
		    continue;
		}
		return -1;
	    case -1:
	        if( errno == EINTR) continue;
	        if( verbose) fprintf( stderr, "err select(): %s\n", strerror( errno));
	        return -1;
	}
        break;
    }
    return 0;
}
																		    

static int write_str( int fd, char *str, int len) {
  if( verbose) fprintf( stderr, "->[%d]<%s>\n", len, str);

// Send String length first
  int bytesToSend = sizeof(len),
  bytesWritten = 0,
  num = 0;

  for (bytesWritten = 0; bytesWritten < bytesToSend; bytesWritten += num) {
    num = write(fd, (void *)( (uintptr_t) &len + (uintptr_t)bytesWritten ), bytesToSend - bytesWritten);
    if (num <= 0 && errno != EINTR) {
      syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_ERR),"Write stream error: %s. Error=%d", strerror(errno), errno);
      return -1;
    }
  }

  bytesToSend = len;
  bytesWritten = 0;
  num = 0;
       
  for (bytesWritten = 0; bytesWritten < bytesToSend; bytesWritten += num) {
    num = write(fd, (void *)( (uintptr_t)str + (uintptr_t)bytesWritten ), bytesToSend - bytesWritten);
    if (num <= 0 && errno != EINTR) {
      syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_ERR),"Write stream error: %s. Error=%d", strerror(errno), errno);
      return -1;
    }
  }
    
  return len;
}


static int read_str( int fd, char *str, int size) {
  int len;
  int bytesToRead;

// Read len correctly first
  bytesToRead=sizeof( len);
  int bytesReaden = 0;
  int num = 0;

  for (bytesReaden = 0; bytesReaden < bytesToRead; bytesReaden += num) {
    num = read(fd, (void *)( (uintptr_t) &len + (uintptr_t)bytesReaden ), bytesToRead - bytesReaden);
    if (num <= 0 && errno != EINTR) {
      syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_ERR),"Read stream error: %s. Error=%d", strerror(errno), errno);
      return -1;
    }
  }

// Then read string of length len
  bytesToRead = len,
  bytesReaden = 0,
  num = 0;

  for (bytesReaden = 0; bytesReaden < bytesToRead; bytesReaden += num) {
    num = read(fd, (void *)( (uintptr_t)str + (uintptr_t)bytesReaden ), bytesToRead - bytesReaden);
    if (num <= 0 && errno != EINTR) {
      syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_ERR),"Read stream error: %s. Error=%d", strerror(errno), errno);
      return -1;
    }
  }
  if ( bytesReaden < bytesToRead) {
    syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_ERR),"Read stream error: read %d bytes instead of %d", bytesReaden, bytesToRead);
    return -1;
  }

  str[ len] = 0;
  if( verbose) 
    fprintf( stderr, "<-[%d]<%s>\n", len, str);
  return len;
}

static int read_ok( int fd) {
  char str[ 256];
  return (read_str( fd, str, sizeof( str)) == 2 && str[0] == 'O' && str[1] == 'K');
}

static int write_ok( int fd) {
  return write_str( fd, "OK", 2) == 2;
}

