app.TasksPanel = Ext.extend(Ext.Container, {
  title: ~'Задачі'~,
		iconCls: 'x-tab-tasks',
		
		loadTasks: function(){
		  Ext.Ajax.request({
				  params: {
						  cmd: 'tasks',
								history: this.history ? 1 : 0
						},
						scope: this,
						success: function (response, opts) {
								var o = Ext.decode(response.responseText) || {};
								this.storeTasks.loadData(o.queues || {});
								this.storeTasks.loadData(o.grid_queue || {}, true);	
								this.storeTasks.groupBy('hostname', true);
						}																				
				});
		},
		
		initComponent: function() {
			 var self = this;
				
				this.history = false;
				
				this.storeTasks = new Ext.data.GroupingStore({
						groupField: 'hostname',				
						reader: new Ext.data.JsonReader({
								fields: [{
										name: 'hostname',
										mapping: 'H'
								},{
										name: 'end_time', 
										mapping: 'e'
								},{
										name: 'node_list',
										mapping: 'N'
								},{
										name: 'partition', 
										mapping: 'P'
								},{
										name: 'dependency', 
										mapping: 'E'
								},{
										name: 'job_name', 
										mapping: 'j'
								},{
										name: 'user_name', 
										mapping: 'u'
								},{
										name: 'run_time',
										mapping: 'M'
								},{
										name: 'cores_count',
										mapping: 'C',
										type: 'int'
								},{
									 name: 'wait_time',
										mapping: 'W'
								},{
										name: 'start_time', 
										mapping: 'b'
								},{
										name: 'status',
										mapping: 't'
								},{
										name: 'id',
										mapping: 'i'				
								},{
										name: 'job' // for dependency tasks list
								}]
						}),
						listeners: {
								load: function(store, records){
										var i, d;
										for (i = 0; i < records.length; i++) {
												d = records[i].data;
												d.job = d.job_name + ' (' + d.id + '), ' + d.user_name;
												records[i].commit();						
										}		
								}
						}				
				});
			
				var taskStatuses = {'R': ~'Виконується'~, 'PD': ~'Очікує'~, 
																								'CD': ~'Завершена'~, 'CA': ~'Скасована'~, 
																								'F': ~'Завершена з помилкою'~, 'TO': ~'Вийшов час'~, 'NF': ~'Збій вузла'~};
				
				this.gridTasks = new Ext.grid.GridPanel({
						region: 'center',
						store: this.storeTasks,
						colModel: new Ext.grid.ColumnModel({
								defaults: {
										sortable: true
								},
								columns: [{
										header: ~'Кластер'~,
										dataIndex: 'hostname'
								},{
										header: 'ID',
										dataIndex: 'id'
								},{
										header: ~'Розділ'~,
										dataIndex: 'partition'
								},{
										header: ~"Ім'я задачі"~,
										dataIndex: 'job_name'
								},{
										header: ~'Користувач'~,
										dataIndex: 'user_name'
								},{
										header: ~'Стан'~,
										dataIndex: 'status',
										width: 50,
										renderer: function(x){
												var r = taskStatuses[x];
												return Ext.isDefined(r) ? r : x;
										}
								},{
									 header: ~'Час очікування'~,
										dataIndex: 'wait_time',
										align: 'right',
										width: 100,
										renderer: Ext.util.Format.periodFormat
								},{
										header: ~'Час роботи'~,
										dataIndex: 'run_time',
										align: 'right',
										renderer: Ext.util.Format.periodFormat,	
										width: 200					
								},{
										header: ~'Ядер'~,
										dataIndex: 'cores_count',
										align: 'right',
										width: 50
								},{
										xtype: 'actioncolumn',
										menuDisabled: true,
										width: 26,
										getClass: function(v, meta, rec) {  
												var s = rec.get('status');
												if ((s == 'R' || s == 'PD') && (self.interface.userSettings.is_admin || self.interface.userSettings.work_login == rec.get('user_name')))
														return 'queue-delete-icon';
												else
														return '';		
										},
										handler: function(grid, rowIndex, colIndex) {
												var r = grid.getStore().getAt(rowIndex), task_id = r.get('id'), middleware = r.get('middleware');
												Ext.Msg.show({
														title: ~'Черга'~, 
														msg: String.format(~'Скасувати задачу {0}?'~, task_id), 
														icon: Ext.MessageBox.QUESTION,
														buttons: {ok: true, cancel: true},
														task_id: task_id,
														middleware: middleware,
														fn: function (btn, text, opt) {
																if (btn == 'ok') {
																		Ext.Ajax.request({
																				params: {
																						cmd: 'cancel',
																						job_id: opt.task_id,
																						middleware: opt.middleware
																				},
																				success: function(){
																						self.loadTasks();
																				}
																		});
																}  
														}								
												});
										}		
								}]
						}),
						stripeRows: true,
						view: new Ext.grid.GroupingView({
								forceFit: true,
								hideGroupedColumn: true
						}),
						sm: new Ext.grid.RowSelectionModel({
								singleSelect: true,
								listeners: {
										rowselect: function (sm, rowIdx, r) {
												self.panelInfo.update(r.data);
										}
								}
						}),
						tbar: [{
								text: ~'Оновити'~,
								iconCls: 'x-action-refresh',
								scope: this,
								handler: self.loadTasks
						}, {
								text: ~'Історія запусків'~,
								iconCls: 'x-action-history',
								enableToggle: true,
								scope: this,
								toggleHandler: function(b, pressed) {
										self.history = pressed;
										self.loadTasks();
								}
						}]				
				});		

    this.panelInfo = new Ext.Panel({
						region: 'south',
						title: ~"Додаткова інформація"~,
						height: 200,
						collapsible: true,
						minSize: 150,
						split: true,
					 bodyStyle: {
								padding: '7px'
						},
						autoScroll: true,
						tpl: ['<table cellspacing="0" cellpadding="2" border="0"><tr><td>ID:</td><td> {id}<br />', 
								'<tr><td width="150">Розділ: </td><td> {partition}</td></tr>',
								'<tr><td>Стан: </td><td> {status}</td></tr>',
								'<tr><td>Користувач: </td><td> {user_name}</td></tr>',
								'<tr><td>Ім\'я задачі: </td><td> {job_name}</td></tr>',
								'<tr><td>Час роботи: </td><td> {run_time:periodFormat}</td></tr>',
								'<tr><td>Кількість ядер: </td><td> {cores_count}</td></tr>',
								'<tr><td>Початок роботи: </td><td> {start_time}</td></tr>',
								'<tr><td>Кінець роботи: </td><td> {end_time}</td></tr>',
								'<tr><td valign="top">Список вузлів: </td><td> {node_list}</td></tr>',
								'<tr><td>Залежність: </td><td> {dependency}</td></tr></table>']
						
				});
				
				var config = {
					 layout: 'border',
					 items: [this.gridTasks, this.panelInfo]
				}

    Ext.applyIf(this, config);

    app.TasksPanel.superclass.initComponent.call(this);				

		}
});

Ext.reg('taskspanel', app.TasksPanel);