app.TextViewer = Ext.extend(Ext.form.TextArea, {
  cls: 'x-view-ta',
		mode: 'view',
		readOnly: true,
		
		initComponent: function(){
				var config = {
						fileName: '',
						middleware: '',
						fileSize: 0,
						curPage: 0,
						pagesCount: 1,
						new_file: false,
						taskWatch: {
								run: function(){
										this.mode = 'watch';
										this.setPage(-1);
								},
								scope: this,
								interval: 10000
						}
				};

    Ext.apply(this, config);

    app.TextViewer.superclass.initComponent.call(this);	
    this.addEvents('pageset');	
		},
		
		getFile: function(){
		  return {fileName: this.fileName, middleware: this.middleware};	
		},
		
		setFile: function(fileName, middleware, new_file) {
				this.setValue('');
				this.fileName = fileName;
				this.middleware = middleware;
				if (!new_file)
  				this.setPage(0);
		},
		
		getPage: function() {
		  return this.curPage;
		},
		
		getPagesCount: function() {
			 return this.pagesCount
		},
		
		setPage: function(page) {
				var self = this,
				    chunkSizes = {'view': 100*1024, 'edit': 1024*1024*1024, 'watch': 20*1024},
				    chunkSize = chunkSizes[this.mode]; // TODO: make this better

    page = Math.min(self.pagesCount-1, Math.round(page, 0));
				Ext.Ajax.request({
						page: page,
						params: {
							 cmd: 'less',
								f: self.fileName,
								m: self.middleware,
								o: page*chunkSize,
								s: chunkSize
						},
						success: function(response, opts) {
								var o = Ext.decode(response.responseText) || {}, less = o.less || {};
								self.setValue(less.text);
								self.fileSize = less.filesize;
								self.pagesCount = Math.ceil(self.fileSize/chunkSize);
								self.curPage = Math.min(self.pagesCount-1, opts.page);
								self.setReadOnly(self.mode != 'edit');
								self.fireEvent('pageset', self.mode, self.curPage, self.pagesCount);
								if (self.mode == 'watch') {
      			 var d = self.getEl().dom;
										d.scrollTop = d.scrollHeight;
		      }
						}													
				})		  
		},
		
		toggleEdit: function(x) {
			 this.mode = x ? 'edit' : 'view';
				this.setPage(0);
		},
		
		startWatch: function(){
				Ext.TaskMgr.start(this.taskWatch);
		},
		
		stopWatch: function(){
    Ext.TaskMgr.stop(this.taskWatch);
			 this.mode = 'view';
				this.setPage(0);
		},
		
		afterRender: function(){
		  app.TextViewer.superclass.afterRender.call(this);
				
    this.el.set({spellcheck: 'false'});
		}
});

app.ViewerWindow = Ext.extend(Ext.Window, {
		width: 800,
		height: 500,
		title: ~'Перегляд файла'~,
		iconCls: 'x-action-view',
		closeAction: 'hide',
		modal: true,
		maximizable: true,
		minimizable: true,
		layout: 'fit',

  viewFile: function(o, new_file) {
				new_file = (new_file == true);
				
				this.setTitle(String.format(~'Перегляд файла {0}'~, o.file_name));
							
				this.btnEdit.toggle(new_file); // if new - edit file, else view file
								
			 this.textView.setFile(o.dir + o.file_name, o.middleware, new_file);
				
				this.show();
				this.center();
		},
		
		initComponent: function(){			
				var self = this;
    					
    this.textView = new app.TextViewer({
						listeners: {
							 pageset: function(mode, page, pages){
									 self.slider.setMaxValue(pages);
										self.slider.setValue(page+1);
										self.slider.setDisabled(mode != 'view');
  								self.btnSave.setDisabled(mode != 'edit');	
										self.btnFirst.setDisabled(mode != 'view');
										self.btnPrev.setDisabled(mode != 'view');
										self.btnNext.setDisabled(mode != 'view');
										self.btnLast.setDisabled(mode != 'view');
								}
						}
				});
				
				this.slider = new Ext.Slider({
						minValue: 1,
						maxValue: 1,
						width: 300,
						plugins: new Ext.slider.Tip({
						  getText: function(t){
									 return String.format('{0} / {1}', t.value, t.slider.maxValue);
								}
						}),
						listeners: {
							 changecomplete: function(sl, v){
										self.textView.setPage(v-1);
								}
						}
				});
				
				this.btnEdit = new Ext.Button({
						text: ~'Редагування'~,
						iconCls: 'x-action-edit',
						enableToggle: true,
						toggleGroup: 'mode',
						toggleHandler: function(){
								self.textView.toggleEdit(this.pressed)
						}																																		
    })
				
				this.btnWatch = new Ext.Button({
				  text: ~'Спостереження'~,
						iconCls: 'x-action-tailf',
						enableToggle: true,
						toggleGroup: 'mode',
						toggleHandler: function(){
								this.pressed ? self.textView.startWatch() : self.textView.stopWatch();
						}
				});
				
				this.btnSave = new Ext.Button({
						text: ~'Зберегти'~,
						iconCls: 'x-action-save',
						disabled: true,
						handler: function(){
								var f = self.textView.getFile();
								Ext.Ajax.request({
										params: {
												cmd: 'put',
												c: self.textView.getValue(),
												f: f.fileName,
												m: f.middleware
										},
										success: function(){
										  self.fm.fmActions.refresh.execute(); // refresh files list upon save	
										}
								});
						}																									
		  });
				
				this.btnFirst = new Ext.Button({
					 iconCls: 'x-tbar-page-first',
						handler: function(){
							 self.textView.setPage(0);
						}																															
    });
				
				this.btnPrev = new Ext.Button({
						iconCls: 'x-tbar-page-prev',
						handler: function(){
								var v = self.textView;
								v.setPage(v.getPage() - 1);
						}
				});
				
				this.btnNext = new Ext.Button({
						iconCls: 'x-tbar-page-next',
						handler: function(){
								var v = self.textView;
								v.setPage(v.getPage() + 1);
						}
				});
				
				this.btnLast = new Ext.Button({
						iconCls: 'x-tbar-page-last',
						handler: function(){
							 var v = self.textView;
								v.setPage(v.getPagesCount()-1);
						}																												
				});
				
				var config = {
					 layout: 'fit',
					 items: [this.textView],
				  tbar: [this.btnEdit, this.btnWatch, this.btnSave],
					
						bbar: [this.btnFirst, this.btnPrev, this.slider, this.btnNext, this.btnLast, '->', {
								text: ~'Закрити'~,
								handler: function(){
										self.hide();
								}
						}]
				}
				
    Ext.applyIf(this, config);

    app.ViewerWindow.superclass.initComponent.call(this);				
		}
});